#include <iostream>
#include <vector>
#include <algorithm>
#include <ctime>

using pos_t = uint64_t;
constexpr int board_size = 7;

std::vector<pos_t> square[board_size*board_size];

void init_square() {
    for( int i = 0; i < board_size; ++i ) {
        for( int j = !i; j < board_size; ++j ) {
            for( int x = 0; x < board_size; ++x ) {
                for( int y = 0; y < board_size; ++y ) {
                    const int x0 = x;
                    const int y0 = y;
                    const int x1 = x0 + i;
                    const int y1 = y0 + j;
                    const int x2 = x1 - j;
                    const int y2 = y1 + i;
                    const int x3 = x2 - i;
                    const int y3 = y2 - j;
                    if( x1 < 0 || x1 >= board_size ) { continue; }
                    if( x2 < 0 || x2 >= board_size ) { continue; }
                    if( x3 < 0 || x3 >= board_size ) { continue; }
                    if( y1 < 0 || y1 >= board_size ) { continue; }
                    if( y2 < 0 || y2 >= board_size ) { continue; }
                    if( y3 < 0 || y3 >= board_size ) { continue; }
                    const pos_t z0 = static_cast<pos_t>(1)<<(x0*board_size+y0);
                    const pos_t z1 = static_cast<pos_t>(1)<<(x1*board_size+y1);
                    const pos_t z2 = static_cast<pos_t>(1)<<(x2*board_size+y2);
                    const pos_t z3 = static_cast<pos_t>(1)<<(x3*board_size+y3);
                    const pos_t sq = z0 | z1 | z2 | z3;
                    square[x0*board_size+y0].push_back( sq );
                    square[x1*board_size+y1].push_back( sq );
                    square[x2*board_size+y2].push_back( sq );
                    square[x3*board_size+y3].push_back( sq );
                }
            }
        }
    }
    for( int i = 0; i < board_size*board_size; ++i ) { 
        std::sort( square[i].begin(), square[i].end() );
        square[i].erase( std::unique( square[i].begin(), square[i].end() ), square[i].end() );
    }
}

void display( pos_t board ) {
    for( int x = 0; x < board_size; ++x ) {
        for( int y = 0; y < board_size; ++y ) {
            if( board & static_cast<pos_t>(1)<<(x*board_size+y) ) {
                std::cout << "Z";
            } else {
                std::cout << "@";
            }
        }
        std::cout << std::endl;
    }
    std::cout << "--" << std::endl;
}

bool bb( pos_t pad_board ) {
    for( int x = 0; x < board_size; ++x ) {
        for( int y = 0; y < board_size; ++y ) {
            const pos_t test_stone = static_cast<pos_t>(1)<<(x*board_size+y); 
            if( pad_board & test_stone ) { continue; }
            const pos_t test_board = pad_board | test_stone;
            if( std::any_of( square[x*board_size+y].begin(), square[x*board_size+y].end(), [test_board](pos_t sq){ return ( test_board & sq ) == sq; } ) ) { continue; }
            return false;
        }
    }
    return true;
}

void search(pos_t board) {
    bool answer = true;
    bool make_child = true;
    for( int x = 0; x < board_size; ++x ) {
        for( int y = 0; y < board_size; ++y ) {
            const pos_t new_stone = static_cast<pos_t>(1)<<(x*board_size+y);
            if( board & new_stone ) { make_child = false; continue; } 
            const pos_t new_board = board | new_stone;
            if( std::any_of( square[x*board_size+y].begin(), square[x*board_size+y].end(), [new_board](pos_t sq){ return ( new_board & sq ) == sq; } ) ) { continue; }
            answer = false;
            if( make_child ) {
                if( bb( new_board | ((new_board&(-new_board))-1) ) ) {
                    search( new_board );
                }
            }
        }
    }
    if( answer ) {
        std::cout << board << std::endl;
    }
}

int main() {
    init_square();
    search(static_cast<pos_t>(0));
}

